@@ -919,6 +919,8 @@ class EnvBase(ABC):
         """
         bnt_price = self.external_price_feed['BNT']['price_usd'][-1]
         tkn_price = self.external_price_feed[tkn]['price_usd'][-1]
+        bnt_price = self.validate_type(bnt_price)
+        tkn_price = self.validate_type(tkn_price)
         bnt_rate = self.tokenomics.handle_division(tkn_price, bnt_price)
         return bnt_rate
 
@@ -931,19 +933,46 @@ class EnvBase(ABC):
         dao_msig_initialized_pools = self.dao_msig_initialized_pools
         for TKN in self.whitelisted_tokens:
             BNTperTKN = self.get_bnt_rate(TKN)
-            if (self.available_liquidity_ledger[TKN][TKN][-1] == 0 and self.available_liquidity_ledger[TKN]['BNT'][-1] == 0 and self.vault_ledger[TKN][
-                -1] * BNTperTKN > self.min_liquidity_threshold and (TKN not in self.dao_msig_initialized_pools)) or force:
+            BNTperTKN = self.validate_type(BNTperTKN)
+
+            tkn_tkn_liquidity = self.available_liquidity_ledger[TKN][TKN][-1]
+            tkn_tkn_liquidity = self.validate_type(tkn_tkn_liquidity)
+
+            tkn_bnt_liquidity = self.available_liquidity_ledger[TKN]['BNT'][-1]
+            tkn_bnt_liquidity = self.validate_type(tkn_bnt_liquidity)
+
+            tkn_vault = self.vault_ledger[TKN][-1]
+            tkn_vault = self.validate_type(tkn_vault)
+
+            bnt_vault = self.vault_ledger['BNT'][-1]
+            bnt_vault = self.validate_type(bnt_vault)
+
+            bnt_staking = self.staking_ledger['BNT'][-1]
+            bnt_staking = self.validate_type(bnt_staking)
+
+            bnbnt_pool = self.pool_token_supply_ledger[f"bnBNT_ERC20_contract"]["supply"][-1]
+            bnbnt_pool = self.validate_type(bnbnt_pool)
+
+            funding_limit = self.available_liquidity_ledger[TKN]['funding_limit'][-1]
+            funding_limit = self.validate_type(funding_limit)
+
+            tkn_vault_times_bntpertkn = self.tokenomics.handle_multiplication(tkn_vault, BNTperTKN)
+
+            zero = self.validate_type(0)
+            min_liquidity_threshold = self.validate_type(self.min_liquidity_threshold)
+
+            if (tkn_tkn_liquidity == zero and tkn_bnt_liquidity == zero and tkn_vault_times_bntpertkn > min_liquidity_threshold and (TKN not in self.dao_msig_initialized_pools)) or force:
 
                 self.available_liquidity_ledger[TKN]['block_num'].append(self.available_liquidity_ledger[TKN]['block_num'][-1])
-                self.available_liquidity_ledger[TKN][TKN].append(self.tokenomics.handle_division(self.min_liquidity_threshold, BNTperTKN))
+                self.available_liquidity_ledger[TKN][TKN].append(self.tokenomics.handle_division(min_liquidity_threshold, BNTperTKN))
                 self.available_liquidity_ledger[TKN]['BNT'].append(self.min_liquidity_threshold)
 
-                self.vault_ledger['BNT'].append(self.tokenomics.handle_addition(self.vault_ledger['BNT'][-1], self.min_liquidity_threshold))
-                self.staking_ledger['BNT'].append(self.tokenomics.handle_addition(self.staking_ledger['BNT'][-1], self.min_liquidity_threshold))
-                self.pool_token_supply_ledger[f"bnBNT_ERC20_contract"]["supply"].append(self.tokenomics.handle_addition(self.pool_token_supply_ledger[f"bnBNT_ERC20_contract"]["supply"][-1], self.min_liquidity_threshold))
+                self.vault_ledger['BNT'].append(self.tokenomics.handle_addition(bnt_vault, min_liquidity_threshold))
+                self.staking_ledger['BNT'].append(self.tokenomics.handle_addition(bnt_staking, min_liquidity_threshold))
+                self.pool_token_supply_ledger[f"bnBNT_ERC20_contract"]["supply"].append(self.tokenomics.handle_addition(bnbnt_pool, min_liquidity_threshold))
 
                 self.update_ema(TKN)
-                self.available_liquidity_ledger[TKN]['funding_limit'].append(self.tokenomics.handle_subtraction(self.available_liquidity_ledger[TKN]['funding_limit'][-1], self.min_liquidity_threshold))
+                self.available_liquidity_ledger[TKN]['funding_limit'].append(self.tokenomics.handle_subtraction(funding_limit, min_liquidity_threshold))
                 dao_msig_initialized_pools.append(TKN)
 
         self.dao_msig_initialized_pools = dao_msig_initialized_pools
@@ -960,6 +989,7 @@ class EnvBase(ABC):
         alpha = self.validate_type(self.alpha)
         spot_rate = self.validate_type(self.get_spot_rate(tkn))
         current_ema = self.validate_type(self.ema[tkn]['ema'][-1])
+        ema = self.ema
         if tkn in self.ema:
             if ema[tkn]['block_num'][-1] < self.block_num:
                 ema[tkn]['block_num'].append(self.block_num)
