@@ -8,18 +8,57 @@ class FixedPointTokenomics(TokenomicsBase):
     MAX_UINT256 = 2 ** 256 - 1
     MAX_UINT128 = 2 ** 128 - 1
 
-    """Tokenomics subclass for all floating-point arithmetic.
-        """
+    def handle_unstake_bnt(self, x, exchange_rate, exit_fee, tkn_out=0):
+        self.validate_signage_and_overflow(exchange_rate * x)
+        bnt = exchange_rate * x
+        self.validate_signage_and_overflow(exit_fee * bnt)
+        exit_fee = exit_fee * bnt
+        self.validate_signage_and_overflow(bnt - exit_fee)
+        bnt_out = bnt - exit_fee
+        return tkn_out, bnt_out
+
+    def handle_division(self, numerator, denominator):
+        self.validate_signage_and_overflow(numerator // denominator)
+        exchange_rate = numerator // denominator
+        return exchange_rate
+
+    def handle_multiplication(self, x1, x2):
+        self.validate_signage_and_overflow(x1 * x2)
+        return x1 * x2
+
+    def handle_addition(self, x1, x2):
+        self.validate_signage_and_overflow(x1 + x2)
+        return x1 + x2
+
+    def handle_subtraction(self, x1, x2):
+        self.validate_signage_and_overflow(x1 - x2)
+        return x1 - x2
+
+    def handle_ema(self, alpha, spot_rate, current_ema):
+        x1 = self.handle_multiplication(alpha * spot_rate)
+        x2 = self.handle_subtraction(1, alpha)
+        x3 = self.handle_multiplication(current_ema, x2)
+        new_ema = self.handle_addition(x1, x3)
+        return new_ema
+
+    def get_pool_tokens_issued_to_user(self, x, bnbnt_supply, staked_bnt):
+        self.validate_signage_and_overflow(bnbnt_supply // staked_bnt)
+        exchange_rate = bnbnt_supply // staked_bnt
+        self.validate_signage_and_overflow(exchange_rate * x)
+        pool_tokens_issued_to_user = exchange_rate * x
+        return pool_tokens_issued_to_user
 
     def check_surplus(self, b, c, e, n, is_surplus=False, is_deficit=True, case='check surplus'):
         self.validate_signage_and_overflow(b + c)
         self.validate_signage_and_overflow(1 - n)
+        self.validate_signage_and_overflow((b + c) // (1 - n))
         if (b + c) // (1 - n) > e: is_surplus, is_deficit = True, False
         self.surplus, self.deficit = is_surplus, is_deficit
 
     def check_hlim(self, b, c, e, x):
         self.validate_signage_and_overflow(c * e)
         self.validate_signage_and_overflow(b + c)
+        self.validate_signage_and_overflow((c * e) // (b + c))
         hlim = (c * e) // (b + c)
         satisfies_hlim = hlim > x
         self.hlim, self.satisfies_hlim = hlim, satisfies_hlim
