@@ -15,7 +15,8 @@ class FloatingPointUnstakeTKN(BaseModel):
     n: FloatingPoint
     x: FloatingPoint
     w: FloatingPoint
-
+    staked_bnt: FloatingPoint
+    bnbnt_supply: FloatingPoint
 
 class FixedPointUnstakeTKN(BaseModel):
     a: FixedPoint
@@ -26,3 +27,14 @@ class FixedPointUnstakeTKN(BaseModel):
     n: FixedPoint
     x: FixedPoint
     w: FixedPoint
+    staked_bnt: FixedPoint
+    bnbnt_supply: FixedPoint
+
+FixedPoint = Type[Fxp(None, signed=True, n_int=256, n_frac=18)]
+
+class FixedPointGeneral(BaseModel):
+    val: FixedPoint
+
+class FloatingPointGeneral(BaseModel):
+    val: FloatingPoint
+
